/*
	lpc_efr.c

	original code: SASANO Takayoshi
	--- public domain, no warranty.
*/

#include <basic.h>
#include <bstdio.h>
#include <bstdlib.h>
#include <tcode.h>
#include <driver/pcat/sys.h>

/* please adjust if your machine uses port 0x4e/0x4f to access EFR */
#define	EFER	0x2e
#define	EFIR	0x2e
#define	EFDR	0x2f

/* EFR unlock */
Inline	void	unlock_efr(void)
{
	out_b(EFER, 0x87);
	out_b(EFER, 0x87);

	return;
}

/* EFR lock */
Inline	void	lock_efr(void)
{
	out_b(EFER, 0xaa);

	return;
}

/* read EFR */
LOCAL	UB	read_efr(UB dev, UB reg)
{
	UW	imask;
	UB	dat;

	DI(imask);
	unlock_efr();

	out_b(EFIR, 0x07);
	out_b(EFDR, dev);

	out_b(EFIR, reg);
	dat = in_b(EFDR);

	lock_efr();
	EI(imask);

	return dat;
}

/* write EFR */
LOCAL	UB	write_efr(UB dev, UB reg, UB dat)
{
	UW	imask;

	DI(imask);
	unlock_efr();

	out_b(EFIR, 0x07);
	out_b(EFDR, dev);

	out_b(EFIR, reg);
	out_b(EFDR, dat);

	lock_efr();
	EI(imask);

	return dat;
}

/* TC string -> UW value (max 32bit) */
LOCAL	UW	tcs2hex(const TC *str)
{
	UW	i, d;

	d = 0;

	for (i = 0; i < 8; i++) {
		if (*str == NULL) break;

		d <<= 4;

		if (*str >= TK_0 && *str <= TK_9) d |= *str - TK_0;
		else if (*str >= TK_A && *str <= TK_F) d |= *str - TK_A + 10;
		else if (*str >= TK_a && *str <= TK_f) d |= *str - TK_a + 10;

		str++;
	}

	return d;
}

/* main */
EXPORT	W	main(W ac, TC *av[])
{
	W	err;
	UB	dev, reg, dat;

	if (ac < 3) {
		printf("usage:	%S [dev(hex)] [reg(hex)]\n", av[0]);
		printf("	%S [dev(hex)] [reg(hex)] [dat(hex)]\n", av[0]);
		err = ER_PAR;
		goto fin0;
	}

	dev = tcs2hex(av[1]);
	reg = tcs2hex(av[2]);
	printf("EFR %02x:%02x ", dev, reg);

	if (ac > 3) {
		dat = tcs2hex(av[3]);
		write_efr(dev, reg, dat);
		printf("<-- %02x\n", dat);
	} else {
		printf("--> %02x\n", read_efr(dev, reg));
	}

	err = E_OK;
 fin0:
	return err;
}
